/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -lboost_date_time-gcc42-mt -o date_time1 date_time1.cpp"
// Note that the library name may vary slightly on your system.
// Tested on OpenSUSE 10.3, g++ 4.21, boost 1.34.1 . Note that, by
// default, OpenSUSE 10.3 still comes with boost 1.33.1 . In order to 
// compile Boost 1.34.1 on OpenSUSE 10.3, you might have to add the 
// compiler switch  -fno-strict-aliasing in line 155 of the file 
// $BOOSTHOME/tools/jam/src/build.jam .

#include <boost/date_time.hpp>

using namespace std;
using namespace boost;
using namespace boost::posix_time;
using namespace boost::gregorian;

/****************************************************************/

int main(int argc, char **argv){
  cout << "Test verschiedener Zeitabstaende" << endl;
  ptime t1 = microsec_clock::local_time();
  usleep(100000);
  ptime t2 = microsec_clock::local_time();
  
  ptime t3 = microsec_clock::local_time();
  usleep(200000);
  ptime t4 = microsec_clock::local_time();
  
  time_duration tx = (t2-t1);
  time_duration ty = (t4-t3); 

  cout << "tx=" << tx << ", ty=" << ty << endl;
  
  if(tx >= ty) cout << "tx ist groesser" << endl;
  else cout << "ty ist groesser" << endl;
  
  ptime t5(date(2001, Jan, 1));
  ptime t6(second_clock::local_time());
  cout << "Stunden seit dem 1.1.2001: " << (t6-t5).hours() << endl;
  
  cout << "Zeitintervall aus Stunden, Minuten, Sekunden:" << endl;
  time_duration td = hours(17) + minutes(22) + seconds(3);
  cout << "h:m:s = " << td.hours() << ":" << td.minutes() << ":" << td.seconds() << endl;
  cout << "Gesamt-Sekunden = " << td.total_seconds() << endl;
  
  cout << "Initialisierung mit String \"00:00:00.000\"" << endl;
  time_duration empty(duration_from_string("00:00:00.000"));
  cout << "total_seconds = " << empty.total_seconds() << endl;
}

/****************************************************************/
