/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -o function1 function1.cpp"
// Tested on OpenSUSE 10.3, g++ 4.21, boost 1.34.1 . Note that, by
// default, OpenSUSE 10.3 still comes with boost 1.33.1 . In order to 
// compile Boost 1.34.1 on OpenSUSE 10.3, you might have to add the 
// compiler switch  -fno-strict-aliasing in line 155 of the file 
// $BOOSTHOME/tools/jam/src/build.jam .

#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>

#include <iostream>
#include <string>
#include <vector>

using namespace std;
using namespace boost;

/****************************************************************/

class functionTester
{
public:
    void printMessage(const string& msg){
	if(f_) 
	    f_(msg);
	else  
	    cout << "No callback registered yet" << endl;
    }
    
    void setMessenger(boost::function<void (const string&)> f){	
	f_ = f; 
    }

private:
    boost::function<void (const string&)> f_;
};

/****************************************************************/

void freeMessenger(const string& msg){
    cout << "Free messenger says: " << msg << endl;
}

/****************************************************************/

class messenger
{
public:
    void operator()(const string& msg){
	messagePrinter(msg, 1);
    }

    void messagePrinter(const string& msg, int mode){
	if(mode==1)
	    cout << "Function object says: " << msg << endl;
	else
	    cout << "Member function says: " << msg << endl;
    }
};

/****************************************************************/

int main(int argc, char **argv){
    typedef vector<shared_ptr<functionTester> > ftType;

    ftType ftContainer;

    for(unsigned int i=0; i<4; ++i){
	shared_ptr<functionTester> p(new functionTester);
	ftContainer.push_back(p);
    }

    // Registering a function pointer
    ftContainer[0]->setMessenger(&freeMessenger);
    // Registering a function object
    ftContainer[1]->setMessenger(messenger());
    // Registering a member function with partial argument binding
    messenger Messenger;
    ftContainer[2]->setMessenger(
	boost::bind(&messenger::messagePrinter,
		    &Messenger,_1, 2)
	);

    // Position 3 intentionally left empty

    ftType::iterator it;
    for(it=ftContainer.begin(); it!=ftContainer.end(); ++it)
	(*it)->printMessage("Hello World");

    return 0;
}

/****************************************************************/
