/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -lboost_signals-gcc42-mt -lboost_date_time-gcc42-mt -o signals1 signals1.cpp"
// Note that the library name may vary slightly on your system.
// Tested on OpenSUSE 10.3, g++ 4.21, boost 1.34.1 . Note that, by
// default, OpenSUSE 10.3 still comes with boost 1.33.1 . In order to 
// compile Boost 1.34.1 on OpenSUSE 10.3, you might have to add the 
// compiler switch  -fno-strict-aliasing in line 155 of the file 
// $BOOSTHOME/tools/jam/src/build.jam .

#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/signals.hpp>
#include <boost/date_time.hpp>

#include <vector>
#include <string>

using namespace std;
using namespace boost;
using namespace boost::posix_time;
using namespace boost::gregorian;

/****************************************************************/

class total_time_server
{
public:
  string operator()(void){
    ptime t1(second_clock::local_time());
    return to_simple_string(t1);
  }  
};

/****************************************************************/

class time_difference_server
{
public:
  string operator()(string startDate){
    ptime t1(from_simple_string(startDate));
    ptime t2(second_clock::local_time());
    time_duration td = t2-t1;
    return to_simple_string(td);
  }
};

/****************************************************************/
// aggregate_values is taken from the boost documentation,
// see http://www.boost.org/doc/html/signals/tutorial.html

template<typename Container>
struct aggregate_values
{
  typedef Container result_type;

  template<typename InputIterator>
  Container operator()(InputIterator first, InputIterator last) const {
    return Container(first, last);
  }
};

/****************************************************************/

class time_retriever
{
public:
  boost::signal<string (void)> getTime;
  boost::signal<string (void), aggregate_values<vector<string> > > getAllTimes;
};

/****************************************************************/

int main(int argc, char **argv){
  total_time_server tts;
  time_difference_server tds;
  time_retriever tr;

  boost::signals::connection c=tr.getTime.connect(tts);
  cout << tr.getTime() << endl;
  c.disconnect();

  c=tr.getTime.connect(boost::bind<string>(tds,"2001-01-01"));
  cout << tr.getTime() << endl;
  c.disconnect();

  if(!c.connected()) cout << "Connection terminated!" << endl;

  boost::signals::connection c1=tr.getAllTimes.connect(1,tts);
  boost::signals::connection c2=
    tr.getAllTimes.connect(0,boost::bind<string>(tds,"2001-01-01"));

  vector<string> results = tr.getAllTimes();
  copy(results.begin(), results.end(), 
       ostream_iterator<string>(cout, " "));
  cout << endl;

  return 0;
}

/****************************************************************/
