/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++  -pthread -lboost_thread-gcc42-mt -o thread2 thread2.cpp"
// Note that the library name may vary slightly on your system.
// Tested on OpenSUSE 10.3, g++ 4.21, boost 1.34.1 . Note that, by
// default, OpenSUSE 10.3 still comes with boost 1.33.1 . In order to 
// compile Boost 1.34.1 on OpenSUSE 10.3, you might have to add the 
// compiler switch  -fno-strict-aliasing in line 155 of the file 
// $BOOSTHOME/tools/jam/src/build.jam .

#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

#include <iostream>
#include <string>
#include <vector>

#include "MersenneTwister.h" // included for convenience

using namespace std;
using namespace boost;

#ifndef BOOST_HAS_THREADS
#error "Error: No thread support in this Boost version"
#endif

/****************************************************************/

boost::mutex cnt_mutex;

unsigned int global_store = 0;
const unsigned int MAXCOUNTER = 1000;

void counter(unsigned int n = 1){
  unsigned int tmp_global_store;

  while(true){
    { // additional scope used for scoped_lock
      boost::mutex::scoped_lock lock(cnt_mutex);
      if(global_store >= MAXCOUNTER) break;

      if(++global_store % 100 == 0)
	cout << "In counter function " << n << ": " << global_store << endl;    
      
      tmp_global_store = global_store; // save variable for later usage
    }
    
    // Put the thread to sleep for random amount of time
    // By creating MersenneTwister.h's MTRand random number generator locally, 
    // we can be sure to be thread-safe
    MTRand rnd(tmp_global_store); // initialise with current global store
    xtime xt;
    boost::xtime_get(&xt, boost::TIME_UTC); // the current time
    xt.sec  += 0; // seconds
    xt.nsec += rnd.randInt(1000000); // random amount of nanoseconds in range [0,1000000]
    boost::thread::sleep(xt);
  }
}

/****************************************************************/

class counter_class{
public:
  void operator()(){ counter(2); }
};

/****************************************************************/

int main(int argc, char **argv){
  boost::thread_group thrd_grp;

  counter_class cnt_class;
  boost::function<void (void)> bf_counter = boost::bind(counter, 3);

  thrd_grp.create_thread(&counter);
  thrd_grp.create_thread(cnt_class);
  thrd_grp.create_thread(bf_counter);
  
  thrd_grp.join_all();

  return 0;
}

/****************************************************************/
